@@ -3,14 +3,14 @@
 from .api import init_app
 
 from ..settings import (
-    TIME_LIMIT, OUTPUT_FORMATTER, DEBUG_OUTPUT_FORMATTER, DEBUG, RESPONSE_LOGGER
+    TIME_LIMIT, OUTPUT_FORMATTER, DEBUG_OUTPUT_FORMATTER, DEBUG, RESPONSE_LOGGER, CORS
 )
 
 from ..setup_agent import setup_agent
 from ..core.log import LocalResponseLogger
 
 
-def app_factory(pipeline_configs=None, debug=None, response_time_limit=None):
+def app_factory(pipeline_configs=None, debug=None, response_time_limit=None, cors=None):
     agent, session, workers = setup_agent(pipeline_configs)
     response_logger = LocalResponseLogger(RESPONSE_LOGGER)
     if DEBUG:
@@ -21,7 +21,8 @@ def app_factory(pipeline_configs=None, debug=None, response_time_limit=None):
     app = init_app(
         agent=agent, session=session, consumers=workers,
         logger_stats=response_logger, output_formatter=output_formatter,
-        debug=debug or DEBUG, response_time_limit=response_time_limit or TIME_LIMIT
+        debug=debug or DEBUG, response_time_limit=response_time_limit or TIME_LIMIT,
+        cors=CORS if cors is None else cors
     )
 
     return app
