@@ -43,7 +43,7 @@ class ApiHandler:
                 return web.json_response({})
 
             response = await asyncio.shield(
-                register_msg(utterance=payload, user_telegram_id=user_id,
+                register_msg(utterance=payload, user_external_id=user_id,
                              user_device_type=data.pop('user_device_type', 'http'),
                              date_time=datetime.now(),
                              location=data.pop('location', ''),
@@ -54,6 +54,7 @@ class ApiHandler:
 
             if response is None:
                 raise RuntimeError('Got None instead of a bot response.')
+
             return web.json_response(self.output_formatter(response['dialog'].to_dict()))
 
     async def dialog(self, request):
@@ -68,10 +69,31 @@ class ApiHandler:
 
     async def dialogs_by_user(self, request):
         state_manager = request.app['agent'].state_manager
-        user_telegram_id = request.match_info['user_telegram_id']
-        dialogs = await state_manager.get_dialogs_by_user_ext_id(user_telegram_id)
+        user_external_id = request.match_info['user_external_id']
+        dialogs = await state_manager.get_dialogs_by_user_ext_id(user_external_id)
         return web.json_response([i.to_dict() for i in dialogs])
 
+    async def dialog_rating(self, request):
+        state_manager = request.app['agent'].state_manager
+        data = await request.json()
+        dialog_id = data.pop('dialog_id')
+        user_id = data.pop('user_id', None)
+        rating = data.pop('rating')
+        await state_manager.set_rating_dialog(user_id, dialog_id, rating)
+        return web.Response()
+
+    async def utterance_rating(self, request):
+        state_manager = request.app['agent'].state_manager
+        data = await request.json()
+        user_id = data.pop('user_id', None)
+        rating = data.pop('rating')
+        utt_id = data.pop('utt_id')
+        await state_manager.set_rating_utterance(user_id, utt_id, rating)
+        return web.Response()
+
+    async def options(self, request):
+        return web.Response(headers={'Access-Control-Allow-Methods': 'POST, OPTIONS'})
+
 
 class PagesHandler:
     def __init__(self, debug=False):
@@ -80,6 +102,9 @@ class PagesHandler:
     async def ping(self, request):
         return web.json_response("pong")
 
+    async def options(self, request):
+        return web.Response(headers={'Access-Control-Allow-Methods': 'GET, OPTIONS'})
+
 
 class WSstatsHandler:
     def __init__(self):
@@ -101,6 +126,9 @@ class WSstatsHandler:
 
         return ws
 
+    async def options(self, request):
+        return web.Response(headers={'Access-Control-Allow-Methods': 'GET, OPTIONS'})
+
 
 class WSChatHandler:
     def __init__(self, output_formatter):
@@ -129,7 +157,7 @@ class WSChatHandler:
                     continue
 
                 response = await register_msg(
-                    utterance=payload, user_telegram_id=user_id,
+                    utterance=payload, user_external_id=user_id,
                     user_device_type=data.pop('user_device_type', 'websocket'),
                     date_time=datetime.now(),
                     location=data.pop('location', ''),
@@ -145,3 +173,6 @@ class WSChatHandler:
                 break
 
         return ws
+
+    async def options(self, request):
+        return web.Response(headers={'Access-Control-Allow-Methods': 'GET, OPTIONS'})
